# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .tool import Tool
from ..._models import BaseModel

__all__ = ["ResponseToolSearchOutputItemParam"]


class ResponseToolSearchOutputItemParam(BaseModel):
    tools: List[Tool]
    """The loaded tool definitions returned by the tool search output."""

    type: Literal["tool_search_output"]
    """The item type. Always `tool_search_output`."""

    id: Optional[str] = None
    """The unique ID of this tool search output."""

    call_id: Optional[str] = None
    """The unique ID of the tool search call generated by the model."""

    execution: Optional[Literal["server", "client"]] = None
    """Whether tool search was executed by the server or by the client."""

    status: Optional[Literal["in_progress", "completed", "incomplete"]] = None
    """The status of the tool search output."""
