# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["BetaResponseCompactionItemParamParam", "Agent"]


class Agent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class BetaResponseCompactionItemParamParam(TypedDict, total=False):
    """
    A compaction item generated by the [`v1/responses/compact` API](https://platform.openai.com/docs/api-reference/responses/compact).
    """

    encrypted_content: Required[str]
    """The encrypted content of the compaction summary."""

    type: Required[Literal["compaction"]]
    """The type of the item. Always `compaction`."""

    id: Optional[str]
    """The ID of the compaction item."""

    agent: Optional[Agent]
    """The agent that produced this item."""
